<?php
namespace MFC\Responsiveimagesbackport\ContentObject;

class ContentObjectRenderer extends \TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer {
	/**
	 * Returns a <img> tag with the image file defined by $file and processed according to the properties in the TypoScript array.
	 * Mostly this function is a sub-function to the IMAGE function which renders the IMAGE cObject in TypoScript.
	 * This function is called by "$this->cImage($conf['file'], $conf);" from IMAGE().
	 *
	 * @param string $file File TypoScript resource
	 * @param array $conf TypoScript configuration properties
	 * @return string <img> tag, (possibly wrapped in links and other HTML) if any image found.
	 * @access private
	 * @see IMAGE()
	 * @todo Define visibility
	 */
	public function cImage($file, $conf) {
		$info = $this->getImgResource($file, $conf['file.']);
		$GLOBALS['TSFE']->lastImageInfo = $info;
		if (is_array($info)) {
			if (\TYPO3\CMS\Core\Utility\GeneralUtility::isAllowedAbsPath(PATH_site . $info['3'])) {
				$source = \TYPO3\CMS\Core\Utility\GeneralUtility::rawUrlEncodeFP(\TYPO3\CMS\Core\Utility\GeneralUtility::png_to_gif_by_imagemagick($info[3]));
				$source = $GLOBALS['TSFE']->absRefPrefix . $source;
			} else {
				$source = $info[3];
			}

			// This array is used to collect the image-refs on the page...
			$GLOBALS['TSFE']->imagesOnPage[] = $source;
			$altParam = $this->getAltParam($conf);
			if ($conf['params'] && !isset($conf['params.'])) {
				$params = ' ' . $conf['params'];
			} else {
				$params = isset($conf['params.']) ? ' ' . $this->stdWrap($conf['params'], $conf['params.']) : '';
			}

			$layoutKey = $this->stdWrap($conf['layoutKey'], $conf['layoutKey.']);
			$imageTagTemplate = $this->getImageTagTemplate($layoutKey, $conf);
			$sourceCollection = $this->getImageSourceCollection($layoutKey, $conf, $file);

			$imageTagValues = array(
				'width' => $info[0],
				'height' => $info[1],
				'src' => htmlspecialchars($source),
				'params' => $params,
				'altParams' => $altParam,
				'border' => $this->getBorderAttr(' border="' . intval($conf['border']) . '"'),
				'sourceCollection' => $sourceCollection,
				'selfClosingTagSlash' => (!empty($GLOBALS['TSFE']->xhtmlDoctype) ? ' /' : ''),
			);

			$theValue = $this->substituteMarkerArray($imageTagTemplate, $imageTagValues, '###|###', TRUE, TRUE);

			$linkWrap = isset($conf['linkWrap.']) ? $this->stdWrap($conf['linkWrap'], $conf['linkWrap.']) : $conf['linkWrap'];
			if ($linkWrap) {
				$theValue = $this->linkWrap($theValue, $linkWrap);
			} elseif ($conf['imageLinkWrap']) {
				$theValue = $this->imageLinkWrap($theValue, $info['origFile'], $conf['imageLinkWrap.']);
			}
			$wrap = isset($conf['wrap.']) ? $this->stdWrap($conf['wrap'], $conf['wrap.']) : $conf['wrap'];
			if ($wrap) {
				$theValue = $this->wrap($theValue, $conf['wrap']);
			}
			return $theValue;
		}
	}

	/**
	 * Returns the html-template for rendering the image-Tag if no template is defined via typoscript the
	 * default <img> tag template is returned
	 *
	 * @param string $layoutKey rendering key
	 * @param array $conf TypoScript configuration properties
	 * @return string
	 */
	public function getImageTagTemplate($layoutKey, $conf) {
		if ($layoutKey && isset($conf['layout.']) && isset($conf['layout.'][$layoutKey . '.'])) {
			$imageTagLayout = $this->stdWrap($conf['layout.'][$layoutKey . '.']['element'], $conf['layout.'][$layoutKey . '.']['element.']);
		} else {
			$imageTagLayout = '<img src="###SRC###" width="###WIDTH###" height="###HEIGHT###" ###PARAMS### ###ALTPARAMS### ###BORDER######SELFCLOSINGTAGSLASH###>';
		}
		return $imageTagLayout;
	}

	/**
	 * Render alternate sources for the image tag. If no source collection is given an empty string is returned.
	 *
	 * @param string $layoutKey rendering key
	 * @param array $conf TypoScript configuration properties
	 * @param string $file
	 * @throws \UnexpectedValueException
	 * @return string
	 */
	public function getImageSourceCollection($layoutKey, $conf, $file) {
		$sourceCollection = '';
		if ($layoutKey && $conf['sourceCollection.'] && ($conf['layout.'][$layoutKey . '.']['source'] || $conf['layout.'][$layoutKey . '.']['source.'])) {

			// find active sourceCollection
			$activeSourceCollections = array();
			foreach ($conf['sourceCollection.'] as $sourceCollectionKey => $sourceCollectionConfiguration) {
				if (substr($sourceCollectionKey, -1) == '.') {
					if (
						(isset($sourceCollectionConfiguration['if.']) && $this->checkIf($sourceCollectionConfiguration['if.']))
						|| !isset($sourceCollectionConfiguration['if.'])
					) {
						$activeSourceCollections[] = $sourceCollectionConfiguration;
					}
				}
			}

			// apply option split to configurations
			$srcLayoutOptionSplitted = $GLOBALS['TSFE']->tmpl->splitConfArray($conf['layout.'][$layoutKey . '.'], count($activeSourceCollections));

			// render sources
			foreach ($activeSourceCollections as $key => $sourceConfiguration) {
				$sourceLayout = $this->stdWrap($srcLayoutOptionSplitted[$key]['source'], $srcLayoutOptionSplitted[$key]['source.']);

				$sourceRenderConfiguration = array (
					'file' => $file,
					'file.' => $conf['file.']
				);

				if (isset($sourceConfiguration['pixelDensity'])) {
					$pixelDensity = (int) $this->stdWrap($sourceConfiguration['pixelDensity'], $sourceConfiguration['pixelDensity.']);
				} else {
					$pixelDensity = 1;
				}
				$dimensionKeys = array('width', 'height', 'maxW', 'minW', 'maxH', 'minH');
				foreach ($dimensionKeys as $dimensionKey) {
					$dimension = $this->stdWrap($sourceConfiguration[$dimensionKey], $sourceConfiguration[$dimensionKey . '.']);
					if (!$dimension && isset($conf['file.'][$dimensionKey])) {
						$dimension = $this->stdWrap($conf['file.'][$dimensionKey], $conf['file.'][$dimensionKey . '.']);
					}
					if ($dimension) {
						if (strstr($dimension, 'c') !== FALSE && ($dimensionKey === 'width' || $dimensionKey === 'height')) {
							$dimensionParts = explode('c', $dimension, 2);
							$dimension =  intval($dimensionParts[0] * $pixelDensity) . 'c';
							if ($dimensionParts[1]) $dimension .= $dimensionParts[1];
						} else {
							$dimension = intval($dimension * $pixelDensity);
						}
						$sourceRenderConfiguration['file.'][$dimensionKey] = $dimension;
					}
				}

				$sourceInfo = $this->getImgResource($sourceRenderConfiguration['file'], $sourceRenderConfiguration['file.']);
				$sourceConfiguration['width'] = $sourceInfo[0];
				$sourceConfiguration['height'] = $sourceInfo[1];
				$sourceConfiguration['src'] = htmlspecialchars($sourceInfo[3]);
				$sourceConfiguration['selfClosingTagSlash'] = (!empty($GLOBALS['TSFE']->xhtmlDoctype) ? ' /' : '');

				$oneSourceCollection = $this->substituteMarkerArray($sourceLayout, $sourceConfiguration, '###|###', TRUE, TRUE);

				if (is_array($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['tslib/class.tslib_content.php']['getImageSourceCollection'])) {
					foreach ($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['tslib/class.tslib_content.php']['getImageSourceCollection'] as $classData) {
						/** @var $hookObject \Tx_Responsiveimagesbackport_ContentObject_ContentObjectOneSourceCollectionHookInterface */
						$hookObject = \TYPO3\CMS\Core\Utility\GeneralUtility::getUserObj($classData);
						if (!$hookObject instanceof \Tx_Responsiveimagesbackport_ContentObject_ContentObjectOneSourceCollectionHookInterface) {
							throw new \UnexpectedValueException(
								'$hookObject must implement interface \\Tx_Responsiveimagesbackport_ContentObject_ContentObjectOneSourceCollectionHookInterface',
								1380007853
							);
						}
						$oneSourceCollection = $hookObject->getOneSourceCollection((array) $sourceRenderConfiguration, (array) $sourceConfiguration, $oneSourceCollection, $this);
					}
				}

				$sourceCollection .= $oneSourceCollection;
			}
		}
		return $sourceCollection;
	}
}

?>